/*---------------------------------------------------------------------------------

	default ARM7 core

	Copyright (C) 2005
		Michael Noland (joat)
		Jason Rogers (dovoto)
		Dave Murphy (WinterMute)

	This software is provided 'as-is', without any express or implied
	warranty.  In no event will the authors be held liable for any
	damages arising from the use of this software.

	Permission is granted to anyone to use this software for any
	purpose, including commercial applications, and to alter it and
	redistribute it freely, subject to the following restrictions:

	1.	The origin of this software must not be misrepresented; you
		must not claim that you wrote the original software. If you use
		this software in a product, an acknowledgment in the product
		documentation would be appreciated but is not required.
	2.	Altered source versions must be plainly marked as such, and
		must not be misrepresented as being the original software.
	3.	This notice may not be removed or altered from any source
		distribution.

---------------------------------------------------------------------------------*/

#include <nds.h>


touchPosition first, tempPos;


void VcountHandler() {
	static int lastbut = -1;
	uint16 but = 0, x = 0, y = 0, xpx = 0, ypx = 0, z1 = 0, z2 = 0;

	but = REG_KEYXY;
	if (!((but ^ lastbut) & (1 << 6))) {
		tempPos = touchReadXY();
		if ((tempPos.x == 0) || (tempPos.y == 0)) {
			but |= (1 << 6);
			lastbut = but;
		}
		else {
			x = tempPos.x;
			y = tempPos.y;
			xpx = tempPos.px;
			ypx = tempPos.py;
			z1 = tempPos.z1;
			z2 = tempPos.z2;
		}
		
	}
	else {
		lastbut = but;
		but |= (1 << 6);
	}
	IPC->touchX	= x;
	IPC->touchY	= y;
	IPC->touchXpx = xpx;
	IPC->touchYpx = ypx;
	IPC->touchZ1 = z1;
	IPC->touchZ2 = z2;
	IPC->buttons = but;
}


void VblankHandler(void) {
}


#define  BYTE_QUEUE_BUFFER_SIZE  128
uint8 byte_queue_buffer[BYTE_QUEUE_BUFFER_SIZE];
uint16 byte_queue_input_index = 0;
uint16 byte_queue_output_index = 0;


int byte_queue_full(void) {
	uint16 aux_input = byte_queue_input_index;
	uint16 aux_output;
	if (byte_queue_input_index > byte_queue_output_index)
		aux_output = byte_queue_output_index + BYTE_QUEUE_BUFFER_SIZE;
	else
		aux_output = byte_queue_output_index;
	return (((aux_output - 1) == aux_input) ? 1 : 0);
}

int byte_queue_empty(void) {
	return ((byte_queue_input_index == byte_queue_output_index) ? 1 : 0);
}

int16 byte_queue_pop(void) {
	int16 ret = -1;
	if (!byte_queue_empty()) {
		ret = (int16) byte_queue_buffer[byte_queue_output_index];
		byte_queue_output_index = (byte_queue_output_index + 1) % BYTE_QUEUE_BUFFER_SIZE;
	}
	return ret;
}

void byte_queue_push(uint8 v) {
	if (byte_queue_full())
		return;
	byte_queue_buffer[byte_queue_input_index] = v;
	byte_queue_input_index = (byte_queue_input_index + 1) % BYTE_QUEUE_BUFFER_SIZE;
}

#define  START_BIT  0x5A
#define  STOP_BIT   0x50
#define  NEXT_BYTE  0xFF
int16 bit_index = NEXT_BYTE;
int16 current_byte = 0;

#define  SEND_BIT(v)  (*SRAM = (v))

void timer0_handler(void) {
	if (bit_index == NEXT_BYTE) {
		current_byte = byte_queue_pop();
		if (current_byte < 0)
			return;
		bit_index = START_BIT;
	}
	if (bit_index == START_BIT) {
		SEND_BIT(0);
		bit_index = 0;
	}
	else if (bit_index == STOP_BIT) {
		SEND_BIT(1);
		bit_index = NEXT_BYTE;
	}
	else {
		SEND_BIT((current_byte >> bit_index) & 0x01);
		if (bit_index == 7)
			bit_index = STOP_BIT;
		else
			bit_index++;
	}
}


void fifo_receive_handler(void) {
	uint32 received_data = REG_IPC_FIFO_RX;
	byte_queue_push((uint8) (received_data & 0x000000FF));
}


int main(int argc, char **argv) {
	// read user settings from firmware
	readUserSettings();
	irqInit();
	REG_EXMEMSTAT = ~ARM7_OWNS_ROM;
	SetYtrigger(80);
	irqSet(IRQ_VCOUNT, VcountHandler);
	irqSet(IRQ_VBLANK, VblankHandler);
	// set timer 0 to send MIDI data (31250Hz)
	TIMER_DATA(0) = TIMER_FREQ(31250);
	TIMER_CR(0) = TIMER_ENABLE | TIMER_DIV_1 | TIMER_IRQ_REQ;
	irqSet(IRQ_TIMER0, timer0_handler);
	// set fifo receive interrupt
	REG_IPC_FIFO_CR = IPC_FIFO_ENABLE | IPC_FIFO_RECV_IRQ;
	irqSet(IRQ_FIFO_NOT_EMPTY, fifo_receive_handler);
	irqEnable(IRQ_VBLANK | IRQ_VCOUNT | IRQ_TIMER0 | IRQ_FIFO_NOT_EMPTY);
	// keep the ARM7 mostly idle
	SEND_BIT(1);
	while (1)
		swiWaitForVBlank();
}

